/*
 * Copyright (c) 2003 Benedikt Meurer <benedikt.meurer@unix-ag.uni-siegen.de>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#ifdef HAVE_SYS_WAIT_H
#include <sys/wait.h>
#endif

#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

#include <libxfce4util/i18n.h>
#include <libxfcegui4/dialogs.h>
#include <gtk/gtk.h>

#include "shutdown.h"

#define BORDER		6

/* whether to confirm logout */
gboolean	shutdownConfirm = TRUE;

/* whether to autosave session on logout */
gboolean	shutdownAutoSave = FALSE;

/* default action on logout */
gint		shutdownDefault = SHUTDOWN_LOGOUT;

/*
 */
gboolean
shutdownDialog(gint *shutdownType, gboolean *saveSession)
{
	const gchar *username;
	GtkWidget *dialog;
	GtkWidget *label;
	GtkWidget *dbox;
	GtkWidget *hbox;
	GtkWidget *vbox;
	GtkWidget *image;
	GtkWidget *menu;
	GtkWidget *omenu;
	GtkWidget *mitem;
	GtkWidget *checkbox;
	gint result;
	gchar *text;

	g_return_val_if_fail(saveSession != NULL, FALSE);
	g_return_val_if_fail(shutdownType != NULL, FALSE);

	/*
	 * The user do not want to see a "shutdown dialog", so lets assume,
	 * he's willing to execute the default (whatever that be)
	 */
	if (!shutdownConfirm) {
		*saveSession = shutdownAutoSave;
		*shutdownType = shutdownDefault;
		return(TRUE);
	}

	dialog = gtk_dialog_new_with_buttons(_("Logout session"),
			NULL, GTK_DIALOG_MODAL, 
			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
			GTK_STOCK_OK, GTK_RESPONSE_OK,
			NULL);
	gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER_ALWAYS);
	gtk_window_set_resizable(GTK_WINDOW(dialog), FALSE);

	dbox = GTK_DIALOG(dialog)->vbox;

	/* if we cannot determine user's realname, fallback to account name */
	if ((username = g_get_real_name()) == NULL || !strlen(username))
		username = g_get_user_name();
	text = g_strdup_printf(_("End session for <b>%s</b>"), username);
	label = gtk_label_new(NULL);
	gtk_label_set_markup(GTK_LABEL(label), text);
	gtk_box_pack_start(GTK_BOX(dbox), label, FALSE, TRUE, BORDER);
	gtk_widget_show(label);
	g_free(text);

	/* */
	hbox = gtk_hbox_new(FALSE, 0);
	gtk_box_pack_start(GTK_BOX(dbox), hbox, TRUE, TRUE, BORDER);
	gtk_widget_show(hbox);

	/* */
	image = gtk_image_new_from_stock(GTK_STOCK_QUIT, GTK_ICON_SIZE_DIALOG);
	gtk_box_pack_start(GTK_BOX(hbox), image, FALSE, TRUE, BORDER);
	gtk_widget_show(image);

	/* */
	vbox = gtk_vbox_new(FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, BORDER);
	gtk_widget_show(vbox);

	/* */
	label = gtk_label_new(_("What do you want to do next?"));
	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, TRUE, BORDER);
	gtk_widget_show(label);

	/* */
	menu = gtk_menu_new();

	/* */
	mitem = gtk_menu_item_new_with_mnemonic(_("_Quit current session"));
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), mitem);
	gtk_widget_show(mitem);

	/* */
	mitem = gtk_menu_item_new_with_mnemonic(_("_Reboot the computer"));
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), mitem);
	gtk_widget_show(mitem);

	/* */
	mitem = gtk_menu_item_new_with_mnemonic(_("_Turn off computer"));
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), mitem);
	gtk_widget_show(mitem);

	/* */
	omenu = gtk_option_menu_new();
	gtk_option_menu_set_menu(GTK_OPTION_MENU(omenu), menu);
	gtk_option_menu_set_history(GTK_OPTION_MENU(omenu), shutdownDefault);
	gtk_box_pack_start(GTK_BOX(vbox), omenu, FALSE, TRUE, BORDER);
	gtk_widget_show(omenu);

	/* */
	checkbox = gtk_check_button_new_with_mnemonic(
			_("_Save session for future logins"));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbox), TRUE);
	gtk_widget_set_sensitive(checkbox, !shutdownAutoSave);
	gtk_box_pack_start(GTK_BOX(vbox), checkbox, FALSE, TRUE, BORDER);
	gtk_widget_show(checkbox);

	result = gtk_dialog_run(GTK_DIALOG(dialog));

	if (result == GTK_RESPONSE_OK) {
		*saveSession = gtk_toggle_button_get_active(
				GTK_TOGGLE_BUTTON(checkbox));
		*shutdownType = gtk_option_menu_get_history(
				GTK_OPTION_MENU(omenu));
	}

	gtk_widget_destroy(dialog);

	/*
	 * run an iteration through the main loop to give the
	 * confirm dialog time to disappear
	 */
	g_main_context_iteration(NULL, TRUE);

	return(result == GTK_RESPONSE_OK);
}

/*
 */
gint
shutdown(gint type)
{
	gchar *command;
	gchar *buf;
	GError *error;
	gint status;

	error = NULL;

	switch (type) {
	case SHUTDOWN_REBOOT:
		command = g_strdup_printf("%s reboot", SHUTDOWN_COMMAND);
		break;

	case SHUTDOWN_HALT:
		command = g_strdup_printf("%s halt", SHUTDOWN_COMMAND);
		break;

	default:
		return(EXIT_SUCCESS);
	}

	if (!g_spawn_command_line_sync(command, NULL, &buf, &status, &error)) {
		xfce_err(_("The following error occured while trying to "
			   "shutdown the computer:\n\n%s"), error->message);
		g_error_free(error);
		g_free(command);
		return(EXIT_FAILURE);
	}

	if (!WIFEXITED(status) || WEXITSTATUS(status) != 0) {
		xfce_err(_(
			"The following error occured while trying to "
			"shutdown the computer:\n\n%s"), buf);
		g_free(command);
		return(EXIT_FAILURE);
	}

	g_free(command);
	return(EXIT_SUCCESS);
}

